#!/usr/bin/env python
import os
import sys
import time
import logging
from watchdog.observers import Observer
from watchdog.events import LoggingEventHandler

# For handle args and watchmysass
from utils import handleArgs, searchNestedDirectories
from WatchMySASS import WatchMySASS

paths = []
rules = []
ARG_DICT = {}

# Create a pausable observer to avoid infinite loop
# https://github.com/gorakhargosh/watchdog/issues/181
class PausingObserver(Observer):
    def dispatch_events(self, *args, **kwargs):
        if not getattr(self, '_is_paused', False):
            super(PausingObserver, self).dispatch_events(*args, **kwargs)

    def pause(self):
        self._is_paused = True

    def resume(self):
        time.sleep(1)  # allow interim events to be queued
        self.event_queue.queue.clear()
        self._is_paused = False

# Create a handler class for the event_handler
# event_handler needs to call a dispatch method with self & event args
class Handler:
    def __init__(self, observer, arg_dict):
        self = self
        self.arg_dict = arg_dict
        self.rules = arg_dict["rules"]
        self.observer = observer

    def dispatch(self, event):
        path, filename = os.path.split(os.path.abspath(event.src_path))
        if self.shouldUpdate(path, filename) == True:
            self.observer.pause()
            # Only compile the changed file
            self.arg_dict["rules"] = [{path: filename}]
            WatchMySASS(self.arg_dict)
            self.observer.resume()

    def shouldUpdate(self, path, filename):
        for rule in self.rules:
            key = rule.keys()[0]
            if key == path and rule[path] == '*':
                return True
            elif key == path and rule[path] == filename:
                if filename.split('.')[-1] == 'html' and filename.split('-')[-1] != 'compiled.html':
                    return True
        return False

###################### Main Function #############################
def observe(argDict):
    logging.basicConfig(level=logging.INFO,
                        format='%(asctime)s - %(message)s',
                        datefmt='%Y-%m-%d %H:%M:%S')

    ARG_DICT = argDict
    paths = argDict["paths"]
    rules = argDict["rules"]

    # Grab files from nested directories
    files = []
    for path in paths:
        files = list(set(searchNestedDirectories(path, '*.html')))

    # Add nested files to rules
    for file in files:
        _path, _filename = os.path.split(file)
        rules.append({_path: _filename})

    # Set observer args
    observer = PausingObserver()
    event_handler = Handler(observer, ARG_DICT)

    # Loop through paths and add to the observer
    for path in paths:
        print('\nListening for HTML/SCSS changes in %s...' % path)
        observer.schedule(event_handler, path, recursive=True)

    # Listen for changes
    observer.start()
    try:
        while True:
            time.sleep(1)
    # Kill on ctrl + c
    except KeyboardInterrupt:
        observer.stop()
    observer.join()



import re
import os
import argparse
from glob import glob

def readFile(fName, printAfter = False):
    f = open(fName,'r')
    if f.mode == 'r':
        contents = f.read()
        if printAfter == True:
            print(contents)
        return contents
    else:
        print('File %s not found.' % fName)

def quickWrite(fName, string, printAfter = False):
    f = open(fName, 'w+') # w+ creates file if not found
    f.write(string)
    f.close()
    if printAfter == True:
        print(string)

def stripNewLines(str):
    stripped = re.sub('\s\s+', ' ', str)
    return stripped

# Recursively grab all specified file extensions from directory and all subdirectories
def searchNestedDirectories(_path, extensions):
    PATH = os.path.abspath(_path)
    matches = [y for x in os.walk(PATH) for y in glob(os.path.join(x[0], extensions))]
    return list(set(matches))

# Handle system arguments when the program is called
def handleArgs():
    temp = []
    rules = []

    # Get the directory to parse
    argParser = argparse.ArgumentParser()
    argParser.add_argument("relpath", type=str, metavar="path/to/compile", help="Path(s) or File(s) to compile", nargs="*")
    # store_true is counter-intuitive
    # this means store true IF present, default is false
    argParser.add_argument("-w", "--watch", help="Continuously watch directory(s)/file(s) for changes", action="store_true")
    argParser.add_argument("-u", "--uncompressed", help="Don't minify CSS output", action="store_true")
    argParser.add_argument("-d", "--destructive", help="Save changes to original file instead of *-compiled.html", action="store_true")
    args = argParser.parse_args()

    # Account for no arguments
    if len(args.relpath) == 0:
        temp.append(os.path.abspath('.'))
        rules.append({os.path.abspath('.'): '*'})

    else:
        for p in args.relpath:
            _path, _filename = os.path.split(os.path.abspath(p))

            # Save path to the list to watch
            if _path not in temp:
                temp.append(_path)

            # If file, save path/file to the rules
            if os.path.isfile(p):
                rules.append({_path: _filename})

            # Otherwise save * as the file for the path/file rule
            else:
                rules.append({_path: '*'})

    # Set the temp list of paths as args.relpath
    args.relpath = temp
    return (args, rules)
